import RemoteApi_pb2
import remoteapi
"""
TestCentering.py

Unit test for Remote API Eye Centering functionality
"""

client = remoteapi.DiscoveryClient().createClient()
try:
    if client is None:
        raise Exception("Failed to connect.")

    pi = client.pluginApi()
    piSimdis = pi.simdis()
    piData = pi.data()
    pi.config().setResponsesEnabled(True)

    # Set up scenario
    f = piData.isScenarioInitialized()
    if not f.isOk(1000):
        raise Exception("Failed isScenarioInitialized")

    if f.value():
        f = piData.deleteScenario()
        if not f.isOk(1000):
            raise Exception("Failed to delete scenario")

    f = piData.initializeScenario("Python Centering Test", 0, 0, 0)
    if not f.isOk(1000):
        raise Exception("Failed to initialize scenario")

    # Create some platforms
    idList = []
    f = piData.createPlatform("Platform 1", "x", 47)
    if not f.isOk(1000):
        raise Exception("Failed to create platform")
    idList.append(f.value())

    f = piData.createPlatform("Platform 2", "x", 88)
    if not f.isOk(1000):
        raise Exception("Failed to create platform")
    idList.append(f.value())

    f = piData.createPlatform("Platform 3", "x", 101)
    if not f.isOk(1000):
        raise Exception("Failed to create platform")
    idList.append(f.value())

    ############################################

    # Center on multiple ids
    msg = RemoteApi_pb2.Command()
    camInfoMsg = msg.piSimdis.view.setCameraInfo
    camInfoMsg.viewport = "MainView"
    camInfoMsg.centroidTether.ids[:] = idList
    f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
    if not f.isOk(1000):
        raise Exception("Failed to setCameraInfo: " + f.responseDescription())

    # Verify centering operation
    f = piSimdis.getCameraInfo()
    if not f.isOk(1000):
        raise Exception("Failed to getCameraInfo: " + f.responseDescription())
    if f.value().viewport != "MainView":
        raise Exception("GetCameraInfo returned incorrect viewport")
    if f.value().centroidTether.ids.__len__() != 3:
        raise Exception("GetCameraInfo wrong number of tether ids: ")

    ############################################

    # Center on a single entity
    msg = RemoteApi_pb2.Command()
    camInfoMsg = msg.piSimdis.view.setCameraInfo
    camInfoMsg.viewport = "MainView"
    camInfoMsg.tetherId = idList[1]
    f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
    if not f.isOk(1000):
        raise Exception("Failed to setCameraInfo: " + f.responseDescription())

    # Verify centering operation
    f = piSimdis.getCameraInfo()
    if not f.isOk(1000):
        raise Exception("Failed to getCameraInfo: " + f.responseDescription())
    if f.value().viewport != "MainView":
        raise Exception("GetCameraInfo returned incorrect viewport")
    if f.value().centroidTether.ids.__len__() != 0:
        raise Exception("GetCameraInfo tetherIds should be empty")
    if f.value().tetherId != idList[1]:
        raise Exception("GetCameraInfo wrong tether entity id")

    ############################################

    # Center on Platforms 1 and 3
    del idList[1]
    msg = RemoteApi_pb2.Command()
    camInfoMsg = msg.piSimdis.view.setCameraInfo
    camInfoMsg.viewport = "MainView"
    camInfoMsg.centroidTether.ids[:] = idList
    f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
    if not f.isOk(1000):
        raise Exception("Failed to setCameraInfo: " + f.responseDescription())

    # Verify centering operation
    f = piSimdis.getCameraInfo()
    if not f.isOk(1000):
        raise Exception("Failed to getCameraInfo: " + f.responseDescription())
    if f.value().viewport != "MainView":
        raise Exception("GetCameraInfo returned incorrect viewport")
    if f.value().centroidTether.ids.__len__() != 2:
        raise Exception("GetCameraInfo wrong number of tether ids")
    if f.value().centroidTether.ids[0] != idList[0]:
        raise Exception("GetCameraInfo incorrect centered id")
    if f.value().centroidTether.ids[1] != idList[1]:
        raise Exception("GetCameraInfo incorrect centered id")

    ############################################

    # Provide a single tether id and multiple tether ids
    # Single tether id should take precedence because it is defined in the message after multiple ids
    msg = RemoteApi_pb2.Command()
    camInfoMsg = msg.piSimdis.view.setCameraInfo
    camInfoMsg.viewport = "MainView"
    camInfoMsg.centroidTether.ids[:] = idList
    camInfoMsg.tetherId = idList[0]
    f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
    if not f.isOk(1000):
        raise Exception("Failed to setCameraInfo: " + f.responseDescription())

    # Verify centering operation
    f = piSimdis.getCameraInfo()
    if not f.isOk(1000):
        raise Exception("Failed to getCameraInfo: " + f.responseDescription())
    if f.value().viewport != "MainView":
        raise Exception("GetCameraInfo returned incorrect viewport")
    if f.value().centroidTether.ids.__len__() != 0:
        raise Exception("GetCameraInfo tetherIds should be empty")
    if f.value().tetherId != idList[0]:
        raise Exception("GetCameraInfo wrong tether entity id")

    ############################################

    # Attempt to center on an entity list with an invalid entry
    idList.append(55) # Invalid ID
    msg = RemoteApi_pb2.Command()
    camInfoMsg = msg.piSimdis.view.setCameraInfo
    camInfoMsg.viewport = "MainView"
    camInfoMsg.centroidTether.ids[:] = idList
    f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
    if f.isOk(1000):
        raise Exception("setCameraInfo passed when it should have failed")

    # All done, successful if we got here
    print("Test successful")
finally:
    client.close()
