import math
import RemoteApi_pb2
import remoteapi
import remoteapi.PluginApi
import time

"""
TestHudWindow.py

Unit test for Remote API HUD Window command functionality
"""

client = remoteapi.DiscoveryClient().createClient()
try:
	if client is None:
		raise Exception("Failed to connect.")

	pi = client.pluginApi()
	piSimdis = pi.simdis()

	f = piSimdis.registerHudWindow('Test Window', 0.5, 0.5)
	if not f.isOk(2500):
		raise Exception("Failed to registerHudWindow: " + f.responseDescription())

	f = piSimdis.getAllHudWindows()
	if not f.isOk(2500):
		raise Exception("Failed to getAllHudWindows: " + f.responseDescription())
	if not 'Test Window' in f.value():
		raise Exception("getAllHudWindows(): Did not find the window we just created.")

	f = piSimdis.setHudWindowPosition('Test Window', 0.6, 0.4)
	if not f.isOk(2500):
		raise Exception("Failed to setHudWindowPosition: " + f.responseDescription())

	f = piSimdis.getHudWindowPosition('Test Window')
	if not f.isOk(2500):
		raise Exception("Failed to getHudWindowPosition: " + f.responseDescription())
	if f.value().xPercent != 0.6 and f.value().yPercent != 0.4:
		raise Exception("Failed to getHudWindowPosition, position is incorrect.")

	f = piSimdis.resetHudWindowPosition('Test Window')
	if not f.isOk(2500):
		raise Exception("Failed to resetHudWindowPosition: " + f.responseDescription())
	f = piSimdis.getHudWindowPosition('Test Window')
	if not f.isOk(2500):
		raise Exception("Failed to getHudWindowPosition: " + f.responseDescription())
	if f.value().xPercent != 0.5 and f.value().yPercent != 0.5:
		raise Exception("Failed to getHudWindowPosition after reset of position, position is incorrect.")

	"""
	This has the potential to remove user settings, so this section is commented out, but
	was tested on a development machine.
	f = piSimdis.resetAllHudWindowPositions()
	if not f.isOk(2500):
		raise Exception("Failed to resetAllHudWindowPositions: " + f.responseDescription())
	"""

	f = piSimdis.setHudWindowSize('Test Window', -1.0, -2.0, 3.0, 4.0)
	if not f.isOk(2500):
		raise Exception("Failed to setHudWindowSize: " + f.responseDescription())

	f = piSimdis.getHudWindowSize('Test Window')
	if not f.isOk(2500):
		raise Exception("Failed to getHudWindowSize: " + f.responseDescription())
	sz = f.value()
	if sz.leftPixel != -1.0 and sz.bottomPixel != -2.0 and sz.rightPixel != 3.0 and sz.topPixel != 4.0:
		raise Exception("Failed to getHudWindowSize, size is incorrect.")

	f = piSimdis.removeHudWindow('Test Window')
	if not f.isOk(2500):
		raise Exception("Failed to removeHudWindow: " + f.responseDescription())

	print("Test Successful")
finally:
	client.close()
