import time
import remoteapi

"""
TestSimdisShader.py

Unit test for Remote API that test shader commands
"""
# Make the wait times variables so that a very large numbers can be selected
# when running the script with valgrind

waitTimeShort = 1000
waitTimeLong = 10000

import RemoteApi_pb2
import remoteapi.PluginApi

def customAddData(piData, id, points):
	msg = RemoteApi_pb2.Command()
	msg.piData.platform.addData.id = id
	msg.piData.platform.addData.frame.system = 3 # LLA
	for inPoint in points:
		point = msg.piData.platform.addData.points.add();
		point.time = inPoint[0]
		point.position.x = inPoint[1]
		point.position.y = inPoint[2]
		point.position.z = inPoint[3]
	return remoteapi.PluginApi.ResponseHandler(piData.client_, piData.client_.send(msg),
		lambda r: r.response.returnCode)

# Adds data points to the given platform
def sendPlatformData(piData, platformId):
	points = []
	for time in range(0, 100 + 1):
		pct = time * 0.01
		# Store points in time, lat, lon, alt
		points.append([time, 1.0 * pct, 1.0 * pct, 1000.0 * pct])
	customAddData(piData, platformId, points)

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piCommon = pi.common()
piSimdis = pi.simdis()

pi.config().setResponsesEnabled(True)

f = piData.isScenarioInitialized()
if not f.isOk(waitTimeShort):
	raise Exception("Failed isScenarioInitialized")

if f.value():
	f = piData.deleteScenario()
	if not f.isOk(waitTimeLong):
		raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Python Test", 0, 0, 0)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to initialize scenario")

f = piData.setOriginLLA(0.1, 0.2, 1000)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set OriginLLA")

f = piData.setReferenceYear(2013)
if not f.isOk(waitTimeShort):
	raise Exception("Failed to set ref year")

# Add Platform
f = piData.createPlatform("Platform 1", "x", 47)
if not f.isOk(waitTimeLong):
	raise Exception("Failed to create platform")
platformId = f.value();

# Add some data points
sendPlatformData(piData, platformId)

# Should be in file mode; finish loading
f = piData.clockAutoConfigureBeginEnd()
if not f.isOk(waitTimeShort):
	raise Exception("Failed to clockAutoConfigureBeginEnd")
# should be empty
if f.value() != 0:
	raise Exception("Failed to clockAutoConfigureBeginEnd")

# Scenario Attach
f = piSimdis.attachShader(0, 0, "ScenarioColorOverride", """#version 140
#pragma vp_entryPoint test_frag
#pragma vp_location fragment_coloring
uniform bool useOverrideColor;

void test_frag(inout vec4 color)
{
  if (useOverrideColor)
  {
    color.r = 1.0;
    color.g = 1.0;
    color.b = 1.0;
    color.a = 1.0;
  }
}
""");
if not f.isOk(waitTimeShort):
	raise Exception("Failed to attachShader")

# Test Boolean
f = piSimdis.setShaderValueB(0, platformId, "useOverrideColor", True);
if not f.isOk(waitTimeShort):
	raise Exception("Failed to setShaderValueB")

msg = RemoteApi_pb2.Command()
camInfoMsg = msg.piSimdis.view.setCameraInfo
camInfoMsg.viewport = "MainView"
camInfoMsg.tetherId = platformId
camInfoMsg.cameraRAE.rangeMeters = 5
f = remoteapi.PluginApi.ResponseHandler(client, client.send(msg), lambda r: r.response.returnCode)
if not f.isOk(1000):
	raise Exception("Failed to setCameraInfo: " + f.responseDescription())

print("Platform should be white")
time.sleep(5)

# Test Boolean
f = piSimdis.setShaderValueB(0, platformId, "useOverrideColor", False);
if not f.isOk(waitTimeShort):
	raise Exception("Failed to setShaderValueB")

print("Platform should be yellow")
time.sleep(5)

# Platform Attach
f = piSimdis.attachShader(1, platformId, "PlatformColorOverride", """#version 140
#pragma vp_entryPoint test_frag
#pragma vp_location fragment_coloring
uniform bool platformUseOverrideColor;

void test_frag(inout vec4 color)
{
  if (platformUseOverrideColor)
  {
    color.r = 0.0;
    color.g = 1.0;
    color.b = 0.0;
    color.a = 1.0;
  }
}
""");
if not f.isOk(waitTimeShort):
	raise Exception("Failed to attachShader")

# Test Boolean
f = piSimdis.setShaderValueB(1, platformId, "platformUseOverrideColor", True);
if not f.isOk(waitTimeShort):
	raise Exception("Failed to setShaderValueB")

print("Platform should be green")
time.sleep(5)

# Test Boolean
f = piSimdis.setShaderValueB(1, platformId, "platformUseOverrideColor", False);
if not f.isOk(waitTimeShort):
	raise Exception("Failed to setShaderValueB")

f = piSimdis.getShaderValueB(1, platformId, "platformUseOverrideColor");
if not f.isOk(waitTimeShort):
	raise Exception("Failed to getShaderValueB")
	
f = piSimdis.getShaderValueB(1, platformId, "ThisShouldFail");
if f.isOk(waitTimeShort):
	raise Exception("getShaderValueB should have failed, but some how did not.")
	
print("Platform should be yellow")
time.sleep(5)

# Test Integer
f = piSimdis.setShaderValueI(1, platformId, "Integer", 3);
if not f.isOk(waitTimeShort):
	raise Exception("Failed to setShaderValueI")

f = piSimdis.getShaderValueI(1, platformId, "Integer");
if not f.isOk(waitTimeShort):
	raise Exception("Failed to getShaderValueI")

# Test Float
f = piSimdis.setShaderValueF(1, platformId, "Float", 2.5);
if not f.isOk(waitTimeShort):
	raise Exception("Failed to setShaderValueF")

f = piSimdis.getShaderValueF(1, platformId, "Float");
if not f.isOk(waitTimeShort):
	raise Exception("Failed to getShaderValueF")

client.close()
print("Test successful")
